#!/usr/bin/perl
use strict;

die 'Usage: perl format_checker.pl $file\n' unless $#ARGV == 0;

my $file = shift @ARGV;
open IN, $file or die "Error reading $file\n$!";

my $errors = 0;
my %ranks;
my %lines;
my $has_scores = 0;
my $no_scores = 0;

while (my $line = <IN>){
    chomp $line;
    if ($line =~ /^(\d+)\t+([a-z ]+)\t+([a-z ]+)(?:\t+([0-9.]+))?$/){
	unless (defined $ranks{$2}){
	    $ranks{$2} = [];
	    $lines{$2} = [];
	}
	push @{$ranks{$2}}, $1;
	push @{$lines{$2}}, $line;
	if ($4){
	    $has_scores++;
	}else{
	    $no_scores++;
	}
    }else{
	print "Error at line: $line\n";
	$errors++;
    }
}

close IN;

if ($has_scores and $no_scores){
    die "$has_scores lines have real-valued scores but $no_scores do not; scores must be provided for all lines or for none.\n";
}

for my $c (keys %ranks){
    my @sorted = sort {$a <=> $b} @{$ranks{$c}};
    my @sorted_lines = @{$lines{$c}}[sort {$ranks{$c}[$a] <=> $ranks{$c}[$b]} 0 .. $#{$lines{$c}}];
    my $expected = 1;
    my $i;
    while ($i <= $#sorted){
	if ($sorted[$i] != $expected){
	    print "Expected to see rank $expected at line: @sorted_lines[$i]\n";
	    $errors++;
	    $expected = $sorted[$i]+1;
	}else{
	    while ($i < $#sorted && $sorted[$i] == $sorted[$i+1]){
		$expected++;
		$i++;
	    }
	}
	$i++;
	$expected++;
    }
}

if ($errors){
    print "$errors lines had errors\n";
}else{
    print "Format is correct\n";
}
    
